import java.io.*;
import java.util.*;
import java.awt.Point;

public class CleaningRobot_Arknave {
    static final int[] dx = {1, 0, -1, 0};
    static final int[] dy = {0, -1, 0, 1};

    public static void main(String[] args) throws IOException {
        Scanner scan = new Scanner(System.in);
        int n = scan.nextInt();
        int m = scan.nextInt();
        int k = scan.nextInt();

        int[][] isDirty = new int[n][m];
        for (int i = 0; i < k; i++) {
            int x = scan.nextInt();
            int y = scan.nextInt();
            isDirty[x-1][y-1] = 1;
        }

        int[][] psum = buildPSum(isDirty);

        int lo = 0;
        int hi = Math.min(n, m) + 1;

        while (lo + 1 < hi) {
            int mid = (lo + hi) / 2;
            if (can(psum, mid)) {
                lo = mid;
            } else {
                hi = mid;
            }
        }

        if (lo == 0)
            lo = -1;

        System.out.println(lo);
    }

    static boolean can(int[][] psum, int x) {
        int n = psum.length - 1;
        int m = psum[0].length - 1;

        int[][] vis = new int[n][m];

        Queue<Point> q = new LinkedList<>();
        for (int i = 0; q.isEmpty() && i + x <= n; i++) {
            for (int j = 0; q.isEmpty() && j + x <= m; j++) {
                if (count(psum, i, j, i + x, j + x) == 0) {
                    vis[i][j] = 1;
                    q.add(new Point(i, j));
                    break;
                }
            }
        }

        assert q.size() <= 1;
        while (!q.isEmpty()) {
            Point p = q.remove();
            for (int d = 0; d < 4; d++) {
                int nx = p.x + dx[d];
                int ny = p.y + dy[d];
                if (0 <= nx && nx + x <= n && 0 <= ny && ny + x <= m && vis[nx][ny] == 0 && count(psum, nx, ny, nx + x, ny + x) == 0) {
                    vis[nx][ny] = 1;
                    q.add(new Point(nx, ny));
                }
            }
        }

        int[][] vissum = buildPSum(vis);

        // now for each open point, make sure we've cleaned it.
        boolean valid = true;
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                if (count(psum, i, j, i + 1, j + 1) == 0) {
                    valid &= count(vissum, Math.max(0, i - x + 1), Math.max(0, j - x + 1), i + 1, j + 1) > 0;
                }
            }
        }

        return valid;
    }

    static int[][] buildPSum(int[][] data) {
        int n = data.length;
        int m = data[0].length;

        int[][] psum = new int[n + 1][m + 1];
        for (int i = 0; i < n; i++) {
            for (int j = 0; j < m; j++) {
                psum[i + 1][j + 1] = data[i][j] + psum[i + 1][j] + psum[i][j + 1] - psum[i][j];
            }
        }

        return psum;
    }

    static int count(int[][] psum, int i1, int j1, int i2, int j2) {
        assert i1 < i2;
        assert j1 < j2;
        return psum[i2][j2] - psum[i1][j2] - psum[i2][j1] + psum[i1][j1];
    }
}
